@@ -255,6 +255,14 @@ _PUBLIC_ void ndr_print_function_debug(ndr_print_function_t fn, const char *name
 	ndr->print = ndr_print_debug_helper;
 	ndr->depth = 1;
 	ndr->flags = 0;
+
+	/* this is a s4 hack until we build up the courage to pass
+	 * this all the way down 
+	 */
+#if _SAMBA_BUILD_ == 4
+	ndr->iconv_convenience = smb_iconv_convenience_init(talloc_autofree_context(), "ASCII", "UTF-8", true);
+#endif
+
 	fn(ndr, name, flags, ptr);
 	talloc_free(ndr);
 }
@@ -276,6 +284,14 @@ _PUBLIC_ char *ndr_print_struct_string(TALLOC_CTX *mem_ctx, ndr_print_fn_t fn, c
 	ndr->print = ndr_print_string_helper;
 	ndr->depth = 1;
 	ndr->flags = 0;
+
+	/* this is a s4 hack until we build up the courage to pass
+	 * this all the way down 
+	 */
+#if _SAMBA_BUILD_ == 4
+	ndr->iconv_convenience = smb_iconv_convenience_init(talloc_autofree_context(), "ASCII", "UTF-8", true);
+#endif
+
 	fn(ndr, name, ptr);
 	ret = talloc_steal(mem_ctx, (char *)ndr->private_data);
 failed:
@@ -339,9 +355,11 @@ _PUBLIC_ void ndr_set_flags(uint32_t *pflags, uint32_t new_flags)
 	/* the big/little endian flags are inter-dependent */
 	if (new_flags & LIBNDR_FLAG_LITTLE_ENDIAN) {
 		(*pflags) &= ~LIBNDR_FLAG_BIGENDIAN;
+		(*pflags) &= ~LIBNDR_FLAG_NDR64;
 	}
 	if (new_flags & LIBNDR_FLAG_BIGENDIAN) {
 		(*pflags) &= ~LIBNDR_FLAG_LITTLE_ENDIAN;
+		(*pflags) &= ~LIBNDR_FLAG_NDR64;
 	}
 	if (new_flags & LIBNDR_FLAG_REMAINING) {
 		(*pflags) &= ~LIBNDR_ALIGN_FLAGS;
@@ -444,7 +462,7 @@ _PUBLIC_ enum ndr_err_code ndr_pull_subcontext_start(struct ndr_pull *ndr,
 
 	case 4: {
 		uint32_t content_size;
-		NDR_CHECK(ndr_pull_uint32(ndr, NDR_SCALARS, &content_size));
+		NDR_CHECK(ndr_pull_uint3264(ndr, NDR_SCALARS, &content_size));
 		if (size_is >= 0 && size_is != content_size) {
 			return ndr_pull_error(ndr, NDR_ERR_SUBCONTEXT, "Bad subcontext (PULL) size_is(%d) mismatch content_size %d", 
 						(int)size_is, (int)content_size);
@@ -529,7 +547,7 @@ _PUBLIC_ enum ndr_err_code ndr_pull_subcontext_start(struct ndr_pull *ndr,
 
 	subndr = talloc_zero(ndr, struct ndr_pull);
 	NDR_ERR_HAVE_NO_MEMORY(subndr);
-	subndr->flags		= ndr->flags;
+	subndr->flags		= ndr->flags & ~LIBNDR_FLAG_NDR64;
 	subndr->current_mem_ctx	= ndr->current_mem_ctx;
 
 	subndr->data = ndr->data + ndr->offset;
@@ -573,7 +591,7 @@ _PUBLIC_ enum ndr_err_code ndr_push_subcontext_start(struct ndr_push *ndr,
 
 	subndr = ndr_push_init_ctx(ndr, ndr->iconv_convenience);
 	NDR_ERR_HAVE_NO_MEMORY(subndr);
-	subndr->flags	= ndr->flags;
+	subndr->flags	= ndr->flags & ~LIBNDR_FLAG_NDR64;
 
 	if (size_is > 0) {
 		NDR_CHECK(ndr_push_zero(subndr, size_is));
@@ -613,7 +631,7 @@ _PUBLIC_ enum ndr_err_code ndr_push_subcontext_end(struct ndr_push *ndr,
 		break;
 
 	case 4: 
-		NDR_CHECK(ndr_push_uint32(ndr, NDR_SCALARS, subndr->offset));
+		NDR_CHECK(ndr_push_uint3264(ndr, NDR_SCALARS, subndr->offset));
 		break;
 
 	case 0xFFFFFC01:
@@ -728,7 +746,7 @@ _PUBLIC_ uint32_t ndr_token_peek(struct ndr_token_list **list, const void *key)
 _PUBLIC_ enum ndr_err_code ndr_pull_array_size(struct ndr_pull *ndr, const void *p)
 {
 	uint32_t size;
-	NDR_CHECK(ndr_pull_uint32(ndr, NDR_SCALARS, &size));
+	NDR_CHECK(ndr_pull_uint3264(ndr, NDR_SCALARS, &size));
 	return ndr_token_store(ndr, &ndr->array_size_list, p, size);
 }
 
@@ -761,12 +779,12 @@ _PUBLIC_ enum ndr_err_code ndr_check_array_size(struct ndr_pull *ndr, void *p, u
 _PUBLIC_ enum ndr_err_code ndr_pull_array_length(struct ndr_pull *ndr, const void *p)
 {
 	uint32_t length, offset;
-	NDR_CHECK(ndr_pull_uint32(ndr, NDR_SCALARS, &offset));
+	NDR_CHECK(ndr_pull_uint3264(ndr, NDR_SCALARS, &offset));
 	if (offset != 0) {
 		return ndr_pull_error(ndr, NDR_ERR_ARRAY_SIZE, 
 				      "non-zero array offset %u\n", offset);
 	}
-	NDR_CHECK(ndr_pull_uint32(ndr, NDR_SCALARS, &length));
+	NDR_CHECK(ndr_pull_uint3264(ndr, NDR_SCALARS, &length));
 	return ndr_token_store(ndr, &ndr->array_length_list, p, length);
 }
 
@@ -838,7 +856,8 @@ _PUBLIC_ enum ndr_err_code ndr_pull_struct_blob(const DATA_BLOB *blob, TALLOC_CT
 	struct ndr_pull *ndr;
 	ndr = ndr_pull_init_blob(blob, mem_ctx, iconv_convenience);
 	NDR_ERR_HAVE_NO_MEMORY(ndr);
-	NDR_CHECK(fn(ndr, NDR_SCALARS|NDR_BUFFERS, p));
+	NDR_CHECK_FREE(fn(ndr, NDR_SCALARS|NDR_BUFFERS, p));
+	talloc_free(ndr);
 	return NDR_ERR_SUCCESS;
 }
 
@@ -852,12 +871,13 @@ _PUBLIC_ enum ndr_err_code ndr_pull_struct_blob_all(const DATA_BLOB *blob, TALLO
 	struct ndr_pull *ndr;
 	ndr = ndr_pull_init_blob(blob, mem_ctx, iconv_convenience);
 	NDR_ERR_HAVE_NO_MEMORY(ndr);
-	NDR_CHECK(fn(ndr, NDR_SCALARS|NDR_BUFFERS, p));
+	NDR_CHECK_FREE(fn(ndr, NDR_SCALARS|NDR_BUFFERS, p));
 	if (ndr->offset < ndr->data_size) {
 		return ndr_pull_error(ndr, NDR_ERR_UNREAD_BYTES,
 				      "not all bytes consumed ofs[%u] size[%u]",
 				      ndr->offset, ndr->data_size);
 	}
+	talloc_free(ndr);
 	return NDR_ERR_SUCCESS;
 }
 
@@ -871,8 +891,9 @@ _PUBLIC_ enum ndr_err_code ndr_pull_union_blob(const DATA_BLOB *blob, TALLOC_CTX
 	struct ndr_pull *ndr;
 	ndr = ndr_pull_init_blob(blob, mem_ctx, iconv_convenience);
 	NDR_ERR_HAVE_NO_MEMORY(ndr);
-	NDR_CHECK(ndr_pull_set_switch_value(ndr, p, level));
-	NDR_CHECK(fn(ndr, NDR_SCALARS|NDR_BUFFERS, p));
+	NDR_CHECK_FREE(ndr_pull_set_switch_value(ndr, p, level));
+	NDR_CHECK_FREE(fn(ndr, NDR_SCALARS|NDR_BUFFERS, p));
+	talloc_free(ndr);
 	return NDR_ERR_SUCCESS;
 }
 
@@ -887,13 +908,17 @@ _PUBLIC_ enum ndr_err_code ndr_pull_union_blob_all(const DATA_BLOB *blob, TALLOC
 	struct ndr_pull *ndr;
 	ndr = ndr_pull_init_blob(blob, mem_ctx, iconv_convenience);
 	NDR_ERR_HAVE_NO_MEMORY(ndr);
-	NDR_CHECK(ndr_pull_set_switch_value(ndr, p, level));
-	NDR_CHECK(fn(ndr, NDR_SCALARS|NDR_BUFFERS, p));
+	NDR_CHECK_FREE(ndr_pull_set_switch_value(ndr, p, level));
+	NDR_CHECK_FREE(fn(ndr, NDR_SCALARS|NDR_BUFFERS, p));
 	if (ndr->offset < ndr->data_size) {
-		return ndr_pull_error(ndr, NDR_ERR_UNREAD_BYTES,
-				      "not all bytes consumed ofs[%u] size[%u]",
-				      ndr->offset, ndr->data_size);
+		enum ndr_err_code ret;
+		ret = ndr_pull_error(ndr, NDR_ERR_UNREAD_BYTES,
+				     "not all bytes consumed ofs[%u] size[%u]",
+				     ndr->offset, ndr->data_size);
+		talloc_free(ndr);
+		return ret;
 	}
+	talloc_free(ndr);
 	return NDR_ERR_SUCCESS;
 }
 
@@ -1280,6 +1305,7 @@ const static struct {
 	{ NDR_ERR_IPV4ADDRESS, "IPv4 Address Error" },
 	{ NDR_ERR_INVALID_POINTER, "Invalid Pointer" },
 	{ NDR_ERR_UNREAD_BYTES, "Unread Bytes" },
+	{ NDR_ERR_NDR64, "NDR64 assertion error" },
 	{ 0, NULL }
 };
 
