@@ -23,6 +23,7 @@
 #include "lib/netapi/netapi.h"
 #include "lib/netapi/netapi_private.h"
 #include "lib/netapi/libnetapi.h"
+#include "../librpc/gen_ndr/cli_samr.h"
 
 /****************************************************************
 ****************************************************************/
@@ -466,7 +467,7 @@ WERROR NetUserAdd_r(struct libnetapi_ctx *ctx,
 			       &user_handle);
 
  done:
-	if (is_valid_policy_hnd(&user_handle)) {
+	if (is_valid_policy_hnd(&user_handle) && pipe_cli) {
 		rpccli_samr_Close(pipe_cli, ctx, &user_handle);
 	}
 
@@ -770,7 +771,7 @@ static uint32_t samr_acb_flags_to_netapi_flags(uint32_t acb)
 {
 	uint32_t fl = UF_SCRIPT; /* god knows why */
 
-	fl |= ads_acb2uf(acb);
+	fl |= ds_acb2uf(acb);
 
 	return fl;
 }
@@ -1474,10 +1475,10 @@ static WERROR convert_samr_dispinfo_to_NET_DISPLAY(TALLOC_CTX *mem_ctx,
 									  entries_read,
 									  buffer);
 		default:
-			return WERR_UNKNOWN_LEVEL;
+			break;
 	}
 
-	return WERR_OK;
+	return WERR_UNKNOWN_LEVEL;
 }
 
 /****************************************************************
@@ -1696,7 +1697,7 @@ WERROR NetUserGetInfo_r(struct libnetapi_ctx *ctx,
 	}
 
  done:
-	if (is_valid_policy_hnd(&user_handle)) {
+	if (is_valid_policy_hnd(&user_handle) && pipe_cli) {
 		rpccli_samr_Close(pipe_cli, ctx, &user_handle);
 	}
 
@@ -1864,7 +1865,7 @@ WERROR NetUserSetInfo_r(struct libnetapi_ctx *ctx,
 	werr = WERR_OK;
 
  done:
-	if (is_valid_policy_hnd(&user_handle)) {
+	if (is_valid_policy_hnd(&user_handle) && pipe_cli) {
 		rpccli_samr_Close(pipe_cli, ctx, &user_handle);
 	}
 
