@@ -93,7 +93,12 @@ static http_t *cups_connect(TALLOC_CTX *frame)
                 alarm(timeout);
         }
 
+#ifdef HAVE_HTTPCONNECTENCRYPT
+	http = httpConnectEncrypt(server, port, lp_cups_encrypt());
+#else
 	http = httpConnect(server, port);
+#endif
+
 
 	CatchSignal(SIGALRM, SIGNAL_CAST SIG_IGN);
         alarm(0);
@@ -900,6 +905,7 @@ static int cups_job_submit(int snum, struct printjob *pjob)
 	http_t		*http = NULL;		/* HTTP connection to server */
 	ipp_t		*request = NULL,	/* IPP Request */
 			*response = NULL;	/* IPP Response */
+	ipp_attribute_t *attr_job_id = NULL;	/* IPP Attribute "job-id" */
 	cups_lang_t	*language = NULL;	/* Default language */
 	char		uri[HTTP_MAX_URI]; /* printer-uri attribute */
 	const char 	*clientname = NULL; 	/* hostname of client for job-originating-host attribute */
@@ -912,9 +918,10 @@ static int cups_job_submit(int snum, struct printjob *pjob)
 	char *cupsoptions = NULL;
 	char *filename = NULL;
 	size_t size;
+	uint32_t jobid = (uint32_t)-1;
 	char addr[INET6_ADDRSTRLEN];
 
-	DEBUG(5,("cups_job_submit(%d, %p (%d))\n", snum, pjob, pjob->sysjob));
+	DEBUG(5,("cups_job_submit(%d, %p)\n", snum, pjob));
 
        /*
         * Make sure we don't ask for passwords...
@@ -978,12 +985,20 @@ static int cups_job_submit(int snum, struct printjob *pjob)
 	             "job-originating-host-name", NULL,
 		      clientname);
 
+	/* Get the jobid from the filename. */
+	jobid = print_parse_jobid(pjob->filename);
+	if (jobid == (uint32_t)-1) {
+		DEBUG(0,("cups_job_submit: failed to parse jobid from name %s\n",
+				pjob->filename ));
+		jobid = 0;
+	}
+
 	if (!push_utf8_talloc(frame, &jobname, pjob->jobname, &size)) {
 		goto out;
 	}
 	new_jobname = talloc_asprintf(frame,
 			"%s%.8u %s", PRINT_SPOOL_PREFIX,
-			(unsigned int)pjob->smbjob,
+			(unsigned int)jobid,
 			jobname);
 	if (new_jobname == NULL) {
 		goto out;
@@ -1021,6 +1036,13 @@ static int cups_job_submit(int snum, struct printjob *pjob)
 			         ippErrorString(cupsLastError())));
 		} else {
 			ret = 0;
+			attr_job_id = ippFindAttribute(response, "job-id", IPP_TAG_INTEGER);
+			if(attr_job_id) {
+				pjob->sysjob = attr_job_id->values[0].integer;
+				DEBUG(5,("cups_job_submit: job-id %d\n", pjob->sysjob));
+			} else {
+				DEBUG(0,("Missing job-id attribute in IPP response"));
+			}
 		}
 	} else {
 		DEBUG(0,("Unable to print file to `%s' - %s\n", PRINTERNAME(snum),
@@ -1337,14 +1359,12 @@ static int cups_queue_get(const char *sharename,
 	if ((response = cupsDoRequest(http, request, "/")) == NULL) {
 		DEBUG(0,("Unable to get printer status for %s - %s\n", printername,
 			 ippErrorString(cupsLastError())));
-		*q = queue;
 		goto out;
 	}
 
 	if (response->request.status.status_code >= IPP_OK_CONFLICT) {
 		DEBUG(0,("Unable to get printer status for %s - %s\n", printername,
 			 ippErrorString(response->request.status.status_code)));
-		*q = queue;
 		goto out;
 	}
 
@@ -1372,13 +1392,14 @@ static int cups_queue_get(const char *sharename,
 	        fstrcpy(status->message, msg);
 	}
 
+ out:
+
        /*
         * Return the job queue...
 	*/
 
 	*q = queue;
 
- out:
 	if (response)
 		ippDelete(response);
 
