@@ -20,12 +20,7 @@
 #ifndef _PARAM_H /* _PARAM_H */
 #define _PARAM_H 
 
-struct param_opt {
-	struct param_opt *prev, *next;
-	char *key;
-	char *value;
-	int priority;
-};
+#include "../lib/util/parmlist.h"
 
 struct param_context {
 	struct param_section *sections;
@@ -34,7 +29,7 @@ struct param_context {
 struct param_section {
 	const char *name;
 	struct param_section *prev, *next;
-	struct param_opt *parameters;
+	struct parmlist *parameters;
 };
 
 struct param_context;
@@ -334,11 +329,11 @@ struct gensec_settings *lp_gensec_settings(TALLOC_CTX *, struct loadparm_context
 /* The following definitions come from param/generic.c  */
 
 struct param_section *param_get_section(struct param_context *ctx, const char *name);
-struct param_opt *param_section_get(struct param_section *section, 
+struct parmlist_entry *param_section_get(struct param_section *section, 
 				    const char *name);
-struct param_opt *param_get (struct param_context *ctx, const char *name, const char *section_name);
+struct parmlist_entry *param_get (struct param_context *ctx, const char *name, const char *section_name);
 struct param_section *param_add_section(struct param_context *ctx, const char *section_name);
-struct param_opt *param_get_add(struct param_context *ctx, const char *name, const char *section_name);
+struct parmlist_entry *param_get_add(struct param_context *ctx, const char *name, const char *section_name);
 const char *param_get_string(struct param_context *ctx, const char *param, const char *section);
 int param_set_string(struct param_context *ctx, const char *param, const char *value, const char *section);
 const char **param_get_string_list(struct param_context *ctx, const char *param, const char *separator, const char *section);
@@ -362,6 +357,9 @@ int param_write(struct param_context *ctx, const char *fn);
 bool lp_is_mydomain(struct loadparm_context *lp_ctx, 
 			     const char *domain);
 
+bool lp_is_my_domain_or_realm(struct loadparm_context *lp_ctx, 
+			      const char *domain);
+
 /**
   see if a string matches either our primary or one of our secondary 
   netbios aliases. do a case insensitive match
@@ -434,6 +432,8 @@ const char *lp_messaging_path(TALLOC_CTX *mem_ctx,
 struct smb_iconv_convenience *smb_iconv_convenience_init_lp(TALLOC_CTX *mem_ctx,
 							 struct loadparm_context *lp_ctx);
 
+const char *lp_sam_name(struct loadparm_context *lp_ctx);
+
 /* The following definitions come from lib/version.c  */
 
 const char *samba_version_string(void);
