@@ -21,9 +21,7 @@
 */
 
 #include "includes.h"
-#include "../lib/util/dlinklist.h"
 #include "smb_server/smb_server.h"
-#include "librpc/gen_ndr/ndr_misc.h"
 #include "ntvfs/ntvfs.h"
 #include "libcli/raw/libcliraw.h"
 #include "libcli/raw/raw_proto.h"
